package asw.asw830.client.connector;

import asw.asw830.service.*; 
import asw.asw830.service.rmi.*;
import asw.util.logging.AswLogger;

import java.rmi.registry.*;

import java.util.logging.*;

public class HelloClientProxy implements Hello {
	
	/* riferimento al servizio remoto RMI */ 
	private HelloRmiInterface hello; 
	
	/* logger */ 
	private Logger logger = AswLogger.getInstance().getLogger("asw.asw830.client.connector"); 

	public HelloClientProxy(String registryHost, String helloServiceName) {
        /* crea il security manager */ 
    	if (System.getSecurityManager() == null) {
            System.setSecurityManager(new SecurityManager());
        }
        try {
        	/* cerca un riferimento al servizio remoto */  
            logger.info("Hello Client Proxy: Looking for " + helloServiceName + " on " + registryHost);
            Registry registry = LocateRegistry.getRegistry(registryHost);
            this.hello = (HelloRmiInterface) registry.lookup(helloServiceName);
            logger.info("Hello Client Proxy: " + helloServiceName + " found.");
        } catch (Exception e) {
            logger.info("Hello Client Proxy: Exception " + e.getMessage());
        }
	}
	
	public String sayHello(String name) throws HelloException, asw.asw830.service.RemoteException {
		try {
			String result = hello.sayHello(name);
			logger.info("Hello Client Proxy: hello.sayHello(" + name + ") ==> " + result);
			return result; 
		} catch (HelloException e) {
			/* rilancia l'eccezione */ 
			logger.info("Hello Client Proxy: HelloException: " + e.getMessage());
			throw e; 
		} catch (java.rmi.RemoteException e) {
			/* converte l'eccezione */ 
			logger.info("Hello Client Proxy: received java.rmi.RemoteException: " + e.getMessage());
			throw new asw.asw830.service.RemoteException(e.getMessage()); 
		}
	}

}
