package asw.asw830.counter.client.connector;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.logging.Logger;

import asw.asw830.counter.Counter;
import asw.util.logging.AswLogger;

/* Factory per il servizio Counter. */ 
public class ServiceFactory {
	
	/* registryHost  "localhost" oppure "192.168.56.99" */ 
	/* per default  "localhost" */ 
	private static String registryHost = "localhost"; 

	/* logger */ 
	private Logger logger = AswLogger.getInstance().getLogger("asw.asw830.counter.client.connector"); 

	private static ServiceFactory instance = null;    // per singleton 
    
    public static synchronized ServiceFactory getInstance() { 
        if (instance==null) instance = new ServiceFactory(); 
        return instance; 
    }

    /* Factory method per il servizio counter. */ 
    public Counter getCounter(String[] args) {
    	if (args.length>0) {
    		registryHost = args[0]; 
    	}    	
        Counter counter = null;
        try {
        	/* cerca un riferimento al servizio remoto */  
        	String counterServiceName = "rmi:/asw830/Counter"; 
            logger.info("Main: looking for " + counterServiceName + " on " + registryHost);
            Registry registry = LocateRegistry.getRegistry(registryHost);
            counter = (Counter) registry.lookup(counterServiceName);
            logger.info("CounterClient: " + counterServiceName + " found");
        } catch (RemoteException e) {
        	logger.info("CounterClient: RemoteException: " + e.getMessage());
        } catch (NotBoundException e) {
        	logger.info("CounterClient: NotBoundException: " + e.getMessage());
        }
        return counter; 
    }
}

